import os
import glob
from base import console
import time
from datetime import datetime, timedelta
import subprocess

def wait_timeout(cmd, seconds):
    print cmd
    
    proc = subprocess.Popen(cmd, shell=True)
    #http://stackoverflow.com/questions/1359383/python-run-a-process-and-kill-it-if-it-doesnt-end-within-one-hour
    """Wait for a process to finish, or raise exception after timeout"""
    start = time.time()
    end = start + seconds
    interval = min(seconds / 1000.0, .25)

    while True:
        result = proc.poll()
        if result is not None:
            return result
        if time.time() >= end:
            proc.kill()
            raise RuntimeError("Process timed out")
        time.sleep(interval)

def system(cmd):
    """
    execute a command in the shell
    """
    console.log_command(cmd)
    os.system(cmd)

def Path(*parts):
    """
    construct a normalized path from a list of parts
    """
    return os.path.normpath(os.path.join(*parts))

def mkdir_p(*path):
    system('mkdir -p %s' % (Path(*path),))

def mkdir_p2(dirname):
    """
    helper to create a directory recursively (mkdir -p)
    """
    if not path.isdir(dirname):
        parent = path.dirname(dirname)
        if not path.isdir(parent):
            mkdir_p(parent)
        mkdir(dirname)



def nameonly(absfilename):
    base = os.path.basename(absfilename)
    return base[:base.index('.')]
    
def rm_rf(*path):
    system('rm -Rf %s' % (Path(*path),))

rm = os.remove

def mv(source, target):
    system('mv %s %s' % (source, target))
    
def cp_r(source, target):
    system('cp -R %s %s' % (source, target))
    
def cp(source, target):
    system('cp %s %s' % (source, target))


class FileStore(object):
    """
    conveinience wrapper around filesystem related functionality
    
    """
    
    def __init__(self, *base):
        """
        construct a filestore
        
        base - all operations are relative to this path
        """
        self.base = base
        
    def get(self, *key):
        """
        returns absolute path of ressource given by key
        """
        return Path(*(self.base + key))
        
    def get_fs(self, *key):
        """
        return a FileStore relative to this one
        """
        return FileStore(*(self.base + key))
        
    def exists(self, *key):
        return os.path.exists(self.get(*key))
        
    def isdir(self, *key):
        return os.path.isdir(self.get(*key))

    def isfile(self, *key):
        return os.path.isfile(self.get(*key))
    
    def glob(self, *key):
        """
        return list of files/dirs that match pattern in key (e.g. 'mydir/*.exp')
        """
        return glob.glob(Path(*(self.base + key)))
    
    def read(self, *key):
        """
        return the contents of the file given by key
        """
        with open(self.get(*key), 'r') as fp:
            return fp.read()
    
    def create(self, *key):
        """
        create a directory at the location given by key
        """
        mkdir_p(*(self.base + key))
        return self
    
    def clear(self, *key):
        """
        recursively delete the location given by key
        """
        rm_rf(*(self.base + key))
        return self
    
    def copyto(self, abspath):
        """
        recursively copy this store to abspath
        """
        cp_r(self.get(), abspath)
        return self
        
    def copycontents(self, abspath):
        """
        copy contents of store to abspath
        """
        contents = [Path(self.get(), item) for item in os.listdir(self.get())]
        cp_r(' '.join(contents), abspath)
        
        
    def moveto(self, abspath, recreate_dir=False):
        """
        recursively move this store to abspath
        """
        mv(self.get(), abspath)
        if recreate_dir:
            mkdir_p(self.get())
        return self


class Timer(object):

    def start(self):
        self.t0 = datetime.now()
        return self
        
    def stop(self):
        self.t1 = datetime.now()
        self.elapsed = self.t1 - self.t0
        return self.elapsed




