from datetime import timedelta

def format_time(delta):
    return str(delta.seconds) + '.' + str(delta.microseconds).rjust(6, '0')


def Cursor(object):
    
    def __init__(self, data, colnames):
        self.data = data
        self._colnames = colnames
        
    def __getattr__(self, name):
        return self.data[name]

class Reader(object):

    def __init__(self, filename, header=None, parseHeader=True):
        self.fp = open(filename)
        if header is None and parse_header:
            header = self._parse_header(self.fp.readline())
        self.format = header
        self.colnames = [f[0] for f in self.format]
            
    def _parse_header(self):
        return [
            ('int_num', int),
            ('time', float),
        ]
            
    def close(self):
        self.fp.close()
        

class Writer(object):

    def __init__(self, filename, format=None, mode='w'):
        self.fp = open(filename, mode)

    def write(self, *data):
        strdata = self.convert_line(data)
        self.fp.write(strdata + '\n')
        self.fp.flush()
        
    def convert_line(self, data):
        return '\t\t'.join([self.convert_item(item) for item in data])
    
    def convert_item(self, item):
        if isinstance(item, timedelta):
            return format_time(item)
        return str(item)
        

