from api.fs import FileStore, nameonly
from api.config import PATHS
from integration.featurehouse import *
import unittest2 as unittest
import os
from datetime import timedelta

class TestFeatureHouse(unittest.TestCase):
    
    def setUp(self):
        self.fs = FileStore(PATHS.testing.tmpdir)
        self.ctest = 'composetest'
        self.ctestabs = self.fs.get(self.ctest)
        self.fs.clear(self.ctest)
        self.fs.create(self.ctest)
        self.featurefs = FileStore(PATHS.testing.containment_hierarchy)
        
    def tearDown(self):
        self.fs.clear(self.ctest)
        
        
    def test_config(self):
        self.assertTrue(os.path.isfile(PATHS.featurehouse_jar))
        self.assertTrue(os.path.isdir(PATHS.testing.containment_hierarchy))
        self.assertTrue(os.path.isfile(PATHS.testing.variant))
        self.assertTrue(os.path.isfile(PATHS.testing.modelcnf))
        self.assertTrue(os.path.isdir(PATHS.testing.tmpdir))
        
    def test_compose(self):
        #self.assertFalse(self.featurefs.isdir(nameonly(PATHS.testing.variant)))
        duration = compose(
            PATHS.testing.variant,
            PATHS.testing.containment_hierarchy,
            self.ctestabs,
        )
        self.assertFalse(self.featurefs.isdir(nameonly(PATHS.testing.variant)))
        self.assertTrue(self.fs.isdir(self.ctest))
        self.assertTrue(duration > timedelta(0))
        self.assertTrue(duration < timedelta(0, 10))
