from api.fs import FileStore
from api.config import PATHS
from integration.featurescene import *
import unittest2 as unittest
import os


class TestFeatureScene(unittest.TestCase):
    
    def setUp(self):
        self.ctest = 'scenetest'
        self.fs = FileStore(PATHS.testing.tmpdir, self.ctest)

        self.fs.clear()
        self.fs.create()
        
    def tearDown(self):
        pass
        self.fs.clear()
        
        
    def test_generate_scenario(self):
        label = 'testscene.c'
        outfile = self.fs.get(label)
        generate_scenario(
            Scenario(
                'action1',
                'action2',
                'action3',
            ),
            outfile,
            selected_features = ['A', 'B'],
            action_dict=dict(
                action1='A',
                action2='B',
                action3='C',
            )
        )
        self.assertTrue(self.fs.isfile(label))
        generated = self.fs.read(label)
        self.assertIn('action1()', generated)
        self.assertIn('action2()', generated)
        self.assertNotIn('action3()', generated)
        
    def test_generate_scenario_varenc(self):
        label = 'testscenevarenc.c'
        outfile = self.fs.get(label)
        generate_scenario(
            Scenario(
                'action1',
                'action2',
                'action3',
            ),
            outfile,
            variability_encode=True,
            action_dict=dict(
                action1='A',
                action2='B',
                action3='C',
            )
        )
        self.assertTrue(self.fs.isfile(label))
        generated = self.fs.read(label)
        self.assertIn('if (__SELECTED_FEATURE_A)', generated)
        self.assertIn('if (__SELECTED_FEATURE_B)', generated)
        self.assertIn('if (__SELECTED_FEATURE_C)', generated)


